@@ -1,31 +1,26 @@
 /****************************************************************************
 **
-** Copyright (C) 2014 Digia Plc and/or its subsidiary(-ies).
-** Contact: http://www.qt-project.org/legal
+** Copyright (C) 2016 The Qt Company Ltd.
+** Contact: https://www.qt.io/licensing/
 **
 ** This file is part of the qmake application of the Qt Toolkit.
 **
-** $QT_BEGIN_LICENSE:LGPL21$
+** $QT_BEGIN_LICENSE:GPL-EXCEPT$
 ** Commercial License Usage
 ** Licensees holding valid commercial Qt licenses may use this file in
 ** accordance with the commercial license agreement provided with the
 ** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and Digia. For licensing terms and
-** conditions see http://qt.digia.com/licensing. For further information
-** use the contact form at http://qt.digia.com/contact-us.
+** a written agreement between you and The Qt Company. For licensing terms
+** and conditions see https://www.qt.io/terms-conditions. For further
+** information use the contact form at https://www.qt.io/contact-us.
 **
-** GNU Lesser General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 2.1 or version 3 as published by the Free
-** Software Foundation and appearing in the file LICENSE.LGPLv21 and
-** LICENSE.LGPLv3 included in the packaging of this file. Please review the
-** following information to ensure the GNU Lesser General Public License
-** requirements will be met: https://www.gnu.org/licenses/lgpl.html and
-** http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
-**
-** In addition, as a special exception, Digia gives you certain additional
-** rights. These rights are described in the Digia Qt LGPL Exception
-** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
+** GNU General Public License Usage
+** Alternatively, this file may be used under the terms of the GNU
+** General Public License version 3 as published by the Free Software
+** Foundation with exceptions as appearing in the file LICENSE.GPL3-EXCEPT
+** included in the packaging of this file. Please review the following
+** information to ensure the GNU General Public License requirements will
+** be met: https://www.gnu.org/licenses/gpl-3.0.html.
 **
 ** $QT_END_LICENSE$
 **
@@ -47,7 +42,6 @@ const char _CLCompile[]                         = "ClCompile";
 const char _ItemGroup[]                         = "ItemGroup";
 const char _Link[]                              = "Link";
 const char _Lib[]                               = "Lib";
-const char _ManifestTool[]                      = "ManifestTool";
 const char _Midl[]                              = "Midl";
 const char _ResourceCompile[]                   = "ResourceCompile";
 
@@ -60,7 +54,6 @@ const char _AdditionalManifestDependencies[]    = "AdditionalManifestDependencie
 const char _AdditionalOptions[]                 = "AdditionalOptions";
 const char _AdditionalUsingDirectories[]        = "AdditionalUsingDirectories";
 const char _AllowIsolation[]                    = "AllowIsolation";
-const char _AlwaysAppend[]                      = "AlwaysAppend";
 const char _ApplicationConfigurationMode[]      = "ApplicationConfigurationMode";
 const char _AssemblerListingLocation[]          = "AssemblerListingLocation";
 const char _AssemblerOutput[]                   = "AssemblerOutput";
@@ -87,7 +80,6 @@ const char _CompileAsWinRT[]                    = "CompileAsWinRT";
 const char _ConfigurationType[]                 = "ConfigurationType";
 const char _CPreprocessOptions[]                = "CPreprocessOptions";
 const char _CreateHotpatchableImage[]           = "CreateHotpatchableImage";
-const char _CreateHotPatchableImage[]           = "CreateHotPatchableImage";
 const char _Culture[]                           = "Culture";
 const char _DataExecutionPrevention[]           = "DataExecutionPrevention";
 const char _DebugInformationFormat[]            = "DebugInformationFormat";
@@ -97,7 +89,6 @@ const char _DelaySign[]                         = "DelaySign";
 const char _DeleteExtensionsOnClean[]           = "DeleteExtensionsOnClean";
 const char _DisableLanguageExtensions[]         = "DisableLanguageExtensions";
 const char _DisableSpecificWarnings[]           = "DisableSpecificWarnings";
-const char _DisplayLibrary[]                    = "DisplayLibrary";
 const char _DLLDataFileName[]                   = "DLLDataFileName";
 const char _EmbedManagedResourceFile[]          = "EmbedManagedResourceFile";
 const char _EmbedManifest[]                     = "EmbedManifest";
@@ -171,8 +162,6 @@ const char _ModuleDefinitionFile[]              = "ModuleDefinitionFile";
 const char _MultiProcessorCompilation[]         = "MultiProcessorCompilation";
 const char _Name[]                              = "Name";
 const char _NoEntryPoint[]                      = "NoEntryPoint";
-const char _NullTerminateStrings[]              = "NullTerminateStrings";
-const char _ObjectFiles[]                       = "ObjectFiles";
 const char _ObjectFileName[]                    = "ObjectFileName";
 const char _OmitDefaultLibName[]                = "OmitDefaultLibName";
 const char _OmitFramePointers[]                 = "OmitFramePointers";
@@ -181,7 +170,7 @@ const char _Optimization[]                      = "Optimization";
 const char _OptimizeReferences[]                = "OptimizeReferences";
 const char _OutputDirectory[]                   = "OutputDirectory";
 const char _OutputFile[]                        = "OutputFile";
-const char _PlatformToolSet[]                   = "PlatformToolSet";
+const char _PlatformToolSet[]                   = "PlatformToolset";
 const char _PrecompiledHeader[]                 = "PrecompiledHeader";
 const char _PrecompiledHeaderFile[]             = "PrecompiledHeaderFile";
 const char _PrecompiledHeaderOutputFile[]       = "PrecompiledHeaderOutputFile";
@@ -200,7 +189,6 @@ const char _ProxyFileName[]                     = "ProxyFileName";
 const char _RandomizedBaseAddress[]             = "RandomizedBaseAddress";
 const char _RedirectOutputAndErrors[]           = "RedirectOutputAndErrors";
 const char _RegisterOutput[]                    = "RegisterOutput";
-const char _RemoveObjects[]                     = "RemoveObjects";
 const char _ResourceOutputFileName[]            = "ResourceOutputFileName";
 const char _RuntimeLibrary[]                    = "RuntimeLibrary";
 const char _RuntimeTypeInfo[]                   = "RuntimeTypeInfo";
@@ -216,7 +204,6 @@ const char _StringPooling[]                     = "StringPooling";
 const char _StripPrivateSymbols[]               = "StripPrivateSymbols";
 const char _StructMemberAlignment[]             = "StructMemberAlignment";
 const char _SubSystem[]                         = "SubSystem";
-const char _SupportNobindOfDelayLoadedDLL[]     = "SupportNobindOfDelayLoadedDLL";
 const char _SupportUnloadOfDelayLoadedDLL[]     = "SupportUnloadOfDelayLoadedDLL";
 const char _SuppressCompilerWarnings[]          = "SuppressCompilerWarnings";
 const char _SuppressStartupBanner[]             = "SuppressStartupBanner";
@@ -225,8 +212,6 @@ const char _SwapRunFromNet[]                    = "SwapRunFromNet";
 const char _TargetEnvironment[]                 = "TargetEnvironment";
 const char _TargetMachine[]                     = "TargetMachine";
 const char _TerminalServerAware[]               = "TerminalServerAware";
-const char _TrackerLogDirectory[]               = "TrackerLogDirectory";
-const char _TreatLibWarningAsErrors[]           = "TreatLibWarningAsErrors";
 const char _TreatLinkerWarningAsErrors[]        = "TreatLinkerWarningAsErrors";
 const char _TreatSpecificWarningsAsErrors[]     = "TreatSpecificWarningsAsErrors";
 const char _TreatWarningAsError[]               = "TreatWarningAsError";
@@ -245,8 +230,6 @@ const char _UseOfATL[]                          = "UseOfATL";
 const char _UseOfMfc[]                          = "UseOfMfc";
 const char _UseUnicodeForAssemblerListing[]     = "UseUnicodeForAssemblerListing";
 const char _ValidateAllParameters[]             = "ValidateAllParameters";
-const char _VCCustomBuildTool[]                 = "VCCustomBuildTool";
-const char _Verbose[]                           = "Verbose";
 const char _Version[]                           = "Version";
 const char _WarnAsError[]                       = "WarnAsError";
 const char _WarningLevel[]                      = "WarningLevel";
@@ -328,7 +311,7 @@ static QString vcxCommandSeparator()
 static QString unquote(const QString &value)
 {
     QString result = value;
-    result.replace(QStringLiteral("\\\""), QStringLiteral("\""));
+    result.replace(QLatin1String("\\\""), QLatin1String("\""));
     return result;
 }
 
@@ -411,7 +394,7 @@ void VCXProjectWriter::write(XmlOutput &xml, VCProjectSingleConfig &tool)
     xml << decl("1.0", "utf-8")
         << tag("Project")
         << attrTag("DefaultTargets","Build")
-        << attrTag("ToolsVersion", "4.0")
+        << attrTagToolsVersion(tool.Configuration)
         << attrTag("xmlns", "http://schemas.microsoft.com/developer/msbuild/2003")
         << tag("ItemGroup")
         << attrTag("Label", "ProjectConfigurations");
@@ -555,7 +538,7 @@ void VCXProjectWriter::write(XmlOutput &xml, VCProjectSingleConfig &tool)
 
     xmlFilter << decl("1.0", "utf-8")
               << tag("Project")
-              << attrTag("ToolsVersion", "4.0")
+              << attrTagToolsVersion(tool.Configuration)
               << attrTag("xmlns", "http://schemas.microsoft.com/developer/msbuild/2003");
 
     xmlFilter << tag("ItemGroup");
@@ -571,6 +554,7 @@ void VCXProjectWriter::write(XmlOutput &xml, VCProjectSingleConfig &tool)
     addFilters(tempProj, xmlFilter, "Source Files");
     addFilters(tempProj, xmlFilter, "Translation Files");
     addFilters(tempProj, xmlFilter, "Deployment Files");
+    addFilters(tempProj, xmlFilter, "Distribution Files");
 
     for (int x = 0; x < tempProj.ExtraCompilers.count(); ++x)
         addFilters(tempProj, xmlFilter, tempProj.ExtraCompilers.at(x));
@@ -585,11 +569,14 @@ void VCXProjectWriter::write(XmlOutput &xml, VCProjectSingleConfig &tool)
     outputFilter(tempProj, xml, xmlFilter, "Form Files");
     outputFilter(tempProj, xml, xmlFilter, "Resource Files");
     outputFilter(tempProj, xml, xmlFilter, "Deployment Files");
+    outputFilter(tempProj, xml, xmlFilter, "Distribution Files");
 
     for (int x = 0; x < tempProj.ExtraCompilers.count(); ++x) {
         outputFilter(tempProj, xml, xmlFilter, tempProj.ExtraCompilers.at(x));
     }
 
+    outputFilter(tempProj, xml, xmlFilter, "Root Files");
+
     xml << import("Project", "$(VCTargetsPath)\\Microsoft.Cpp.targets");
 
     xml << tag("ImportGroup")
@@ -609,14 +596,12 @@ void VCXProjectWriter::write(XmlOutput &xml, VCProject &tool)
     xml << decl("1.0", "utf-8")
         << tag("Project")
         << attrTag("DefaultTargets","Build")
-        << attrTag("ToolsVersion", "4.0")
+        << attrTagToolsVersion(tool.SingleProjects.first().Configuration)
         << attrTag("xmlns", "http://schemas.microsoft.com/developer/msbuild/2003")
         << tag("ItemGroup")
         << attrTag("Label", "ProjectConfigurations");
 
     bool isWinRT = false;
-    bool isWinPhone = false;
-    bool isWinPhone80 = false; // ### Windows Phone 8.0, remove in Qt 5.4
     for (int i = 0; i < tool.SingleProjects.count(); ++i) {
         xml << tag("ProjectConfiguration")
             << attrTag("Include" , tool.SingleProjects.at(i).Configuration.Name)
@@ -624,8 +609,6 @@ void VCXProjectWriter::write(XmlOutput &xml, VCProject &tool)
             << tagValue("Platform", tool.SingleProjects.at(i).PlatformName)
             << closetag();
         isWinRT = isWinRT || tool.SingleProjects.at(i).Configuration.WinRT;
-        isWinPhone = isWinPhone = tool.SingleProjects.at(i).Configuration.WinPhone;
-        isWinPhone80 = isWinPhone80 || tool.SingleProjects.at(i).Configuration.WinPhone80;
     }
 
     xml << closetag()
@@ -637,17 +620,14 @@ void VCXProjectWriter::write(XmlOutput &xml, VCProject &tool)
 
     if (isWinRT) {
         xml << tagValue("MinimumVisualStudioVersion", tool.Version)
-            << tagValue("DefaultLanguage", "en");
-        if (isWinPhone80) {
-            xml << tagValue("WinMDAssembly", "true");
-            if (tool.SingleProjects.at(0).Configuration.ConfigurationType == typeApplication) {
-                xml << tagValue("XapOutputs", "true");
-                xml << tagValue("XapFilename", "$(RootNamespace)_$(Configuration)_$(Platform).xap");
-            }
-        } else {
-            xml << tagValue("AppContainerApplication", "true")
-                << tagValue("ApplicationType", isWinPhone ? "Windows Phone" : "Windows Store")
-                << tagValue("ApplicationTypeRevision", tool.SdkVersion);
+            << tagValue("DefaultLanguage", "en")
+            << tagValue("AppContainerApplication", "true")
+            << tagValue("ApplicationType", "Windows Store")
+            << tagValue("ApplicationTypeRevision", tool.SdkVersion);
+        if (tool.SdkVersion == "10.0") {
+            const QString ucrtVersion = qgetenv("UCRTVERSION");
+            xml << tagValue("WindowsTargetPlatformVersion", ucrtVersion)
+                << tagValue("WindowsTargetPlatformMinVersion", ucrtVersion);
         }
     }
 
@@ -801,7 +781,7 @@ void VCXProjectWriter::write(XmlOutput &xml, VCProject &tool)
 
     xmlFilter << decl("1.0", "utf-8")
               << tag("Project")
-              << attrTag("ToolsVersion", "4.0")
+              << attrTagToolsVersion(tool.SingleProjects.first().Configuration)
               << attrTag("xmlns", "http://schemas.microsoft.com/developer/msbuild/2003");
 
     xmlFilter << tag("ItemGroup");
@@ -814,6 +794,7 @@ void VCXProjectWriter::write(XmlOutput &xml, VCProject &tool)
     addFilters(tool, xmlFilter, "Source Files");
     addFilters(tool, xmlFilter, "Translation Files");
     addFilters(tool, xmlFilter, "Deployment Files");
+    addFilters(tool, xmlFilter, "Distribution Files");
 
     for (int x = 0; x < tool.ExtraCompilers.count(); ++x)
         addFilters(tool, xmlFilter, tool.ExtraCompilers.at(x));
@@ -828,24 +809,15 @@ void VCXProjectWriter::write(XmlOutput &xml, VCProject &tool)
     outputFilter(tool, xml, xmlFilter, "Form Files");
     outputFilter(tool, xml, xmlFilter, "Resource Files");
     outputFilter(tool, xml, xmlFilter, "Deployment Files");
+    outputFilter(tool, xml, xmlFilter, "Distribution Files");
     for (int x = 0; x < tool.ExtraCompilers.count(); ++x) {
         outputFilter(tool, xml, xmlFilter, tool.ExtraCompilers.at(x));
     }
     outputFilter(tool, xml, xmlFilter, "Root Files");
 
-    if (isWinPhone80) {
-        xml << tag("ItemGroup")
-              << tag("Reference")
-                << attrTag("Include", "platform")
-                << attrTagS("IsWinMDFile", "true")
-                << attrTagS("Private", "false")
-              << closetag()
-            << closetag();
-    }
-
     // App manifest
     if (isWinRT) {
-        QString manifest = isWinPhone80 ? QStringLiteral("WMAppManifest.xml") : QStringLiteral("Package.appxmanifest");
+        const QString manifest = QStringLiteral("Package.appxmanifest");
 
         // Find all icons referenced in the manifest
         QSet<QString> icons;
@@ -866,10 +838,10 @@ void VCXProjectWriter::write(XmlOutput &xml, VCProject &tool)
 
         // Write out manifest + icons as content items
         xml << tag(_ItemGroup)
-            << tag(isWinPhone80 ? "Xml" : "AppxManifest")
+            << tag("AppxManifest")
             << attrTag("Include", manifest)
             << closetag();
-        foreach (const QString &icon, icons) {
+        for (const QString &icon : qAsConst(icons)) {
             xml << tag("Image")
                 << attrTag("Include", icon)
                 << closetag();
@@ -877,12 +849,8 @@ void VCXProjectWriter::write(XmlOutput &xml, VCProject &tool)
         xml << closetag();
     }
 
-    xml << import("Project", "$(VCTargetsPath)\\Microsoft.Cpp.targets");
-
-    if (isWinPhone80)
-        xml << import("Project", "$(MSBuildExtensionsPath)\\Microsoft\\WindowsPhone\\v8.0\\Microsoft.Cpp.WindowsPhone.8.0.targets");
-
-    xml << tag("ImportGroup")
+    xml << import("Project", "$(VCTargetsPath)\\Microsoft.Cpp.targets")
+        << tag("ImportGroup")
         << attrTag("Label", "ExtensionTargets")
         << closetag();
 }
@@ -1175,6 +1143,21 @@ static inline QString toString(subSystemOption option)
     return QString();
 }
 
+static inline QString toString(triState genDebugInfo, linkerDebugOption option)
+{
+    switch (genDebugInfo) {
+    case unset:
+        break;
+    case _False:
+        return "false";
+    case _True:
+        if (option == linkerDebugOptionFastLink)
+            return "DebugFastLink";
+        return "true";
+    }
+    return QString();
+}
+
 static inline QString toString(machineTypeOption option)
 {
     switch (option) {
@@ -1455,7 +1438,6 @@ void VCXProjectWriter::write(XmlOutput &xml, const VCCLCompilerTool &tool)
             << attrTagX(_AdditionalIncludeDirectories, tool.AdditionalIncludeDirectories, ";")
             << attrTagX(_AdditionalOptions, tool.AdditionalOptions, " ")
             << attrTagX(_AdditionalUsingDirectories, tool.AdditionalUsingDirectories, ";")
-//unused    << attrTagS(_AlwaysAppend, tool.AlwaysAppend)
             << attrTagS(_AssemblerListingLocation, tool.AssemblerListingLocation)
             << attrTagS(_AssemblerOutput, toString(tool.AssemblerOutput))
             << attrTagS(_BasicRuntimeChecks, toString(tool.BasicRuntimeChecks))
@@ -1491,7 +1473,6 @@ void VCXProjectWriter::write(XmlOutput &xml, const VCCLCompilerTool &tool)
             << attrTagT(_MinimalRebuild, tool.MinimalRebuild)
             << attrTagT(_MultiProcessorCompilation, tool.MultiProcessorCompilation)
             << attrTagS(_ObjectFileName, tool.ObjectFile)
-//unused    << attrTagX(_ObjectFiles, tool.ObjectFiles, ";")
             << attrTagT(_OmitDefaultLibName, tool.OmitDefaultLibName)
             << attrTagT(_OmitFramePointers, tool.OmitFramePointers)
             << attrTagT(_OpenMPSupport, tool.OpenMP)
@@ -1543,7 +1524,6 @@ void VCXProjectWriter::write(XmlOutput &xml, const VCLinkerTool &tool)
             << attrTagS(_CLRSupportLastError, tool.CLRSupportLastError)
             << attrTagS(_CLRThreadAttribute, tool.CLRThreadAttribute)
             << attrTagT(_CLRUnmanagedCodeCheck, tool.CLRUnmanagedCodeCheck)
-//unused    << attrTagS(_CreateHotPatchableImage, tool.CreateHotPatchableImage)
             << attrTagT(_DataExecutionPrevention, tool.DataExecutionPrevention)
             << attrTagX(_DelayLoadDLLs, tool.DelayLoadDLLs, ";")
             << attrTagT(_DelaySign, tool.DelaySign)
@@ -1553,7 +1533,7 @@ void VCXProjectWriter::write(XmlOutput &xml, const VCLinkerTool &tool)
             << attrTagS(_EntryPointSymbol, tool.EntryPointSymbol)
             << attrTagX(_ForceSymbolReferences, tool.ForceSymbolReferences, ";")
             << attrTagS(_FunctionOrder, tool.FunctionOrder)
-            << attrTagT(_GenerateDebugInformation, tool.GenerateDebugInformation)
+            << attrTagS(_GenerateDebugInformation, toString(tool.GenerateDebugInformation, tool.DebugInfoOption))
             << attrTagT(_GenerateManifest, tool.GenerateManifest)
             << attrTagT(_GenerateWindowsMetadata, tool.GenerateWindowsMetadata)
             << attrTagS(_WindowsMetadataFile, tool.GenerateWindowsMetadata == _True ? tool.WindowsMetadataFile : QString())
@@ -1695,22 +1675,13 @@ void VCXProjectWriter::write(XmlOutput &xml, const VCLibrarianTool &tool)
             << attrTagX(_AdditionalDependencies, tool.AdditionalDependencies, ";")
             << attrTagX(_AdditionalLibraryDirectories, tool.AdditionalLibraryDirectories, ";")
             << attrTagX(_AdditionalOptions, tool.AdditionalOptions, " ")
-//unused    << attrTagS(_DisplayLibrary, tool.DisplayLibrary)
-//unused    << attrTagS(_ErrorReporting, tool.ErrorReporting)
             << attrTagX(_ExportNamedFunctions, tool.ExportNamedFunctions, ";")
             << attrTagX(_ForceSymbolReferences, tool.ForceSymbolReferences, ";")
             << attrTagT(_IgnoreAllDefaultLibraries, tool.IgnoreAllDefaultLibraries)
             << attrTagX(_IgnoreSpecificDefaultLibraries, tool.IgnoreDefaultLibraryNames, ";")
-//unused    << attrTagT(_LinkTimeCodeGeneration, tool.LinkTimeCodeGeneration)
             << attrTagS(_ModuleDefinitionFile, tool.ModuleDefinitionFile)
-//unused    << attrTagS(_Name, tool.Name)
             << attrTagS(_OutputFile, tool.OutputFile)
-//unused    << attrTagX(_RemoveObjects, tool.RemoveObjects, ";")
-//unused    << attrTagS(_SubSystem, tool.SubSystem)
             << attrTagT(_SuppressStartupBanner, tool.SuppressStartupBanner)
-//unused    << attrTagS(_TargetMachine, tool.TargetMachine)
-//unused    << attrTagT(_TreatLibWarningAsErrors, tool.TreatLibWarningAsErrors)
-//unused    << attrTagT(_Verbose, tool.Verbose)
         << closetag(_Lib);
 }
 
@@ -1722,13 +1693,10 @@ void VCXProjectWriter::write(XmlOutput &xml, const VCResourceCompilerTool &tool)
             << attrTagX(_AdditionalOptions, tool.AdditionalOptions, " ")
             << attrTagS(_Culture, toString(tool.Culture))
             << attrTagT(_IgnoreStandardIncludePath, tool.IgnoreStandardIncludePath)
-//unused    << attrTagT(_NullTerminateStrings, tool.NullTerminateStrings)
             << attrTagX(_PreprocessorDefinitions, tool.PreprocessorDefinitions, ";")
             << attrTagS(_ResourceOutputFileName, tool.ResourceOutputFileName)
             << attrTagT(_ShowProgress, toTriState(tool.ShowProgress))
             << attrTagT(_SuppressStartupBanner, tool.SuppressStartupBanner)
-//unused    << attrTagS(_TrackerLogDirectory, tool.TrackerLogDirectory)
-//unused    << attrTagS(_UndefinePreprocessorDefinitions, tool.UndefinePreprocessorDefinitions)
         << closetag(_ResourceCompile);
 }
 
@@ -1935,11 +1903,11 @@ bool VCXProjectWriter::outputFileConfig(OutputFilterData *d, XmlOutput &xml, Xml
                 fileAdded = true;
 
                 xmlFilter << tag("CustomBuild")
-                    << attrTag("Include",Option::fixPathToLocalOS(filename))
+                    << attrTag("Include", Option::fixPathToTargetOS(filename))
                     << attrTagS("Filter", fullFilterName);
 
                 xml << tag("CustomBuild")
-                    << attrTag("Include",Option::fixPathToLocalOS(filename));
+                    << attrTag("Include", Option::fixPathToTargetOS(filename));
 
                 if (filter.Name.startsWith("Form Files")
                         || filter.Name.startsWith("Generated Files")
@@ -1979,11 +1947,13 @@ bool VCXProjectWriter::outputFileConfig(OutputFilterData *d, XmlOutput &xml, Xml
             }
 
             if ( !filter.CompilerTool.PrecompiledHeaderThrough.isEmpty() ) {
-
                 xml << tag("PrecompiledHeaderFile")
                     << attrTag("Condition", condition)
-                    << valueTag(filter.CompilerTool.PrecompiledHeaderThrough)
-                    << tag("PrecompiledHeader")
+                    << valueTag(filter.CompilerTool.PrecompiledHeaderThrough);
+            }
+
+            if (filter.CompilerTool.UsePrecompiledHeader != pchUnset) {
+                xml << tag("PrecompiledHeader")
                     << attrTag("Condition", condition)
                     << valueTag(toString(filter.CompilerTool.UsePrecompiledHeader));
             }
@@ -1996,7 +1966,7 @@ bool VCXProjectWriter::outputFileConfig(OutputFilterData *d, XmlOutput &xml, Xml
 void VCXProjectWriter::outputFileConfig(XmlOutput &xml, XmlOutput &xmlFilter,
                                         const QString &filePath, const QString &filterName)
 {
-    const QString nativeFilePath = Option::fixPathToLocalOS(filePath);
+    const QString nativeFilePath = Option::fixPathToTargetOS(filePath);
     if (filterName.startsWith("Source Files")) {
         xmlFilter << tag("ClCompile")
                   << attrTag("Include", nativeFilePath)
@@ -2056,4 +2026,11 @@ QString VCXProjectWriter::generateCondition(const VCConfiguration &config)
     return QStringLiteral("'$(Configuration)|$(Platform)'=='") + config.Name + QLatin1Char('\'');
 }
 
+XmlOutput::xml_output VCXProjectWriter::attrTagToolsVersion(const VCConfiguration &config)
+{
+    if (config.CompilerVersion >= NET2013)
+        return noxml();
+    return attrTag("ToolsVersion", "4.0");
+}
+
 QT_END_NAMESPACE
